@@ -13,43 +13,52 @@
 """Helper classes use for fake file system implementation."""
 import io
 import locale
+import os
 import platform
 import stat
 import sys
+import time
 from copy import copy
 from stat import S_IFLNK
-
-import os
+from typing import Union, Optional, Any, AnyStr, overload, cast
 
 IS_PYPY = platform.python_implementation() == 'PyPy'
 IS_WIN = sys.platform == 'win32'
 IN_DOCKER = os.path.exists('/.dockerenv')
 
+AnyPath = Union[AnyStr, os.PathLike]
 
-def is_int_type(val):
+
+def is_int_type(val: Any) -> bool:
     """Return True if `val` is of integer type."""
     return isinstance(val, int)
 
 
-def is_byte_string(val):
+def is_byte_string(val: Any) -> bool:
     """Return True if `val` is a bytes-like object, False for a unicode
     string."""
     return not hasattr(val, 'encode')
 
 
-def is_unicode_string(val):
+def is_unicode_string(val: Any) -> bool:
     """Return True if `val` is a unicode string, False for a bytes-like
     object."""
     return hasattr(val, 'encode')
 
 
-def make_string_path(dir_name):
-    if sys.version_info >= (3, 6):
-        dir_name = os.fspath(dir_name)
-    return dir_name
+@overload
+def make_string_path(dir_name: AnyStr) -> AnyStr: ...
 
 
-def to_string(path):
+@overload
+def make_string_path(dir_name: os.PathLike) -> str: ...
+
+
+def make_string_path(dir_name: AnyPath) -> AnyStr:
+    return cast(AnyStr, os.fspath(dir_name))
+
+
+def to_string(path: Union[AnyStr, Union[str, bytes]]) -> str:
     """Return the string representation of a byte string using the preferred
      encoding, or the string itself if path is a str."""
     if isinstance(path, bytes):
@@ -57,7 +66,20 @@ def to_string(path):
     return path
 
 
-def real_encoding(encoding):
+def to_bytes(path: Union[AnyStr, Union[str, bytes]]) -> bytes:
+    """Return the bytes representation of a string using the preferred
+     encoding, or the byte string itself if path is a byte string."""
+    if isinstance(path, str):
+        return bytes(path, locale.getpreferredencoding(False))
+    return path
+
+
+def join_strings(s1: AnyStr, s2: AnyStr) -> AnyStr:
+    """This is a bit of a hack to satisfy mypy - may be refactored."""
+    return s1 + s2
+
+
+def real_encoding(encoding: Optional[str]) -> Optional[str]:
     """Since Python 3.10, the new function ``io.text_encoding`` returns
     "locale" as the encoding if None is defined. This will be handled
     as no encoding in pyfakefs."""
@@ -66,7 +88,24 @@ def real_encoding(encoding):
     return encoding
 
 
-def matching_string(matched, string):
+def now():
+    return time.time()
+
+
+@overload
+def matching_string(matched: bytes, string: AnyStr) -> bytes: ...
+
+
+@overload
+def matching_string(matched: str, string: AnyStr) -> str: ...
+
+
+@overload
+def matching_string(matched: AnyStr, string: None) -> None: ...
+
+
+def matching_string(  # type: ignore[misc]
+        matched: AnyStr, string: Optional[AnyStr]) -> Optional[AnyStr]:
     """Return the string as byte or unicode depending
     on the type of matched, assuming string is an ASCII string.
     """
@@ -82,36 +121,21 @@ class FakeStatResult:
     This is needed as `os.stat_result` has no possibility to set
     nanosecond times directly.
     """
-    _stat_float_times = True
+    def __init__(self, is_windows: bool, user_id: int, group_id: int,
+                 initial_time: Optional[float] = None):
+        self.st_mode: int = 0
+        self.st_ino: Optional[int] = None
+        self.st_dev: int = 0
+        self.st_nlink: int = 0
+        self.st_uid: int = user_id
+        self.st_gid: int = group_id
+        self._st_size: int = 0
+        self.is_windows: bool = is_windows
+        self._st_atime_ns: int = int((initial_time or 0) * 1e9)
+        self._st_mtime_ns: int = self._st_atime_ns
+        self._st_ctime_ns: int = self._st_atime_ns
 
-    def __init__(self, is_windows, user_id, group_id, initial_time=None):
-        self._use_float = None
-        self.st_mode = None
-        self.st_ino = None
-        self.st_dev = None
-        self.st_nlink = 0
-        self.st_uid = user_id
-        self.st_gid = group_id
-        self._st_size = None
-        self.is_windows = is_windows
-        if initial_time is not None:
-            self._st_atime_ns = int(initial_time * 1e9)
-        else:
-            self._st_atime_ns = None
-        self._st_mtime_ns = self._st_atime_ns
-        self._st_ctime_ns = self._st_atime_ns
-
-    @property
-    def use_float(self):
-        if self._use_float is None:
-            return self.stat_float_times()
-        return self._use_float
-
-    @use_float.setter
-    def use_float(self, val):
-        self._use_float = val
-
-    def __eq__(self, other):
+    def __eq__(self, other: Any) -> bool:
         return (
                 isinstance(other, FakeStatResult) and
                 self._st_atime_ns == other._st_atime_ns and
@@ -126,18 +150,17 @@ class FakeStatResult:
                 self.st_mode == other.st_mode
         )
 
-    def __ne__(self, other):
+    def __ne__(self, other: Any) -> bool:
         return not self == other
 
-    def copy(self):
+    def copy(self) -> "FakeStatResult":
         """Return a copy where the float usage is hard-coded to mimic the
         behavior of the real os.stat_result.
         """
         stat_result = copy(self)
-        stat_result.use_float = self.use_float
         return stat_result
 
-    def set_from_stat_result(self, stat_result):
+    def set_from_stat_result(self, stat_result: os.stat_result) -> None:
         """Set values from a real os.stat_result.
         Note: values that are controlled by the fake filesystem are not set.
         This includes st_ino, st_dev and st_nlink.
@@ -150,92 +173,90 @@ class FakeStatResult:
         self._st_mtime_ns = stat_result.st_mtime_ns
         self._st_ctime_ns = stat_result.st_ctime_ns
 
-    @classmethod
-    def stat_float_times(cls, newvalue=None):
-        """Determine whether a file's time stamps are reported as floats
-        or ints.
-
-        Calling without arguments returns the current value.
-        The value is shared by all instances of FakeOsModule.
-
-        Args:
-            newvalue: If `True`, mtime, ctime, atime are reported as floats.
-                Otherwise, they are returned as ints (rounding down).
-        """
-        if newvalue is not None:
-            cls._stat_float_times = bool(newvalue)
-        return cls._stat_float_times
-
     @property
-    def st_ctime(self):
+    def st_ctime(self) -> Union[int, float]:
         """Return the creation time in seconds."""
-        ctime = self._st_ctime_ns / 1e9
-        return ctime if self.use_float else int(ctime)
+        return self._st_ctime_ns / 1e9
 
     @st_ctime.setter
-    def st_ctime(self, val):
+    def st_ctime(self, val: Union[int, float]) -> None:
         """Set the creation time in seconds."""
         self._st_ctime_ns = int(val * 1e9)
 
     @property
-    def st_atime(self):
+    def st_atime(self) -> Union[int, float]:
         """Return the access time in seconds."""
-        atime = self._st_atime_ns / 1e9
-        return atime if self.use_float else int(atime)
+        return self._st_atime_ns / 1e9
 
     @st_atime.setter
-    def st_atime(self, val):
+    def st_atime(self, val: Union[int, float]) -> None:
         """Set the access time in seconds."""
         self._st_atime_ns = int(val * 1e9)
 
     @property
-    def st_mtime(self):
+    def st_mtime(self) -> Union[int, float]:
         """Return the modification time in seconds."""
-        mtime = self._st_mtime_ns / 1e9
-        return mtime if self.use_float else int(mtime)
+        return self._st_mtime_ns / 1e9
 
     @st_mtime.setter
-    def st_mtime(self, val):
+    def st_mtime(self, val: Union[int, float]) -> None:
         """Set the modification time in seconds."""
         self._st_mtime_ns = int(val * 1e9)
 
     @property
-    def st_size(self):
+    def st_size(self) -> int:
         if self.st_mode & S_IFLNK == S_IFLNK and self.is_windows:
             return 0
         return self._st_size
 
     @st_size.setter
-    def st_size(self, val):
+    def st_size(self, val: int) -> None:
         self._st_size = val
 
     @property
-    def st_file_attributes(self):
+    def st_blocks(self) -> int:
+        """Return the number of 512-byte blocks allocated for the file.
+        Assumes a page size of 4096 (matches most systems).
+        Ignores that this may not be available under some systems,
+        and that the result may differ if the file has holes.
+        """
+        if self.is_windows:
+            raise AttributeError(
+                "'os.stat_result' object has no attribute 'st_blocks'")
+        page_size = 4096
+        blocks_in_page = page_size // 512
+        pages = self._st_size // page_size
+        if self._st_size % page_size:
+            pages += 1
+        return pages * blocks_in_page
+
+    @property
+    def st_file_attributes(self) -> int:
         if not self.is_windows:
             raise AttributeError("module 'os.stat_result' "
                                  "has no attribute 'st_file_attributes'")
         mode = 0
         st_mode = self.st_mode
         if st_mode & stat.S_IFDIR:
-            mode |= stat.FILE_ATTRIBUTE_DIRECTORY
+            mode |= stat.FILE_ATTRIBUTE_DIRECTORY  # type:ignore[attr-defined]
         if st_mode & stat.S_IFREG:
-            mode |= stat.FILE_ATTRIBUTE_NORMAL
+            mode |= stat.FILE_ATTRIBUTE_NORMAL  # type:ignore[attr-defined]
         if st_mode & (stat.S_IFCHR | stat.S_IFBLK):
-            mode |= stat.FILE_ATTRIBUTE_DEVICE
+            mode |= stat.FILE_ATTRIBUTE_DEVICE  # type:ignore[attr-defined]
         if st_mode & stat.S_IFLNK:
-            mode |= stat.FILE_ATTRIBUTE_REPARSE_POINT
+            mode |= stat.FILE_ATTRIBUTE_REPARSE_POINT  # type:ignore
         return mode
 
     @property
-    def st_reparse_tag(self):
+    def st_reparse_tag(self) -> int:
         if not self.is_windows or sys.version_info < (3, 8):
             raise AttributeError("module 'os.stat_result' "
                                  "has no attribute 'st_reparse_tag'")
         if self.st_mode & stat.S_IFLNK:
-            return stat.IO_REPARSE_TAG_SYMLINK
+            return stat.IO_REPARSE_TAG_SYMLINK  # type: ignore[attr-defined]
         return 0
 
-    def __getitem__(self, item):
+    def __getitem__(self, item: int) -> Optional[int]:
         """Implement item access to mimic `os.stat_result` behavior."""
         import stat
 
@@ -263,32 +284,32 @@ class FakeStatResult:
         raise ValueError('Invalid item')
 
     @property
-    def st_atime_ns(self):
+    def st_atime_ns(self) -> int:
         """Return the access time in nanoseconds."""
         return self._st_atime_ns
 
     @st_atime_ns.setter
-    def st_atime_ns(self, val):
+    def st_atime_ns(self, val: int) -> None:
         """Set the access time in nanoseconds."""
         self._st_atime_ns = val
 
     @property
-    def st_mtime_ns(self):
+    def st_mtime_ns(self) -> int:
         """Return the modification time in nanoseconds."""
         return self._st_mtime_ns
 
     @st_mtime_ns.setter
-    def st_mtime_ns(self, val):
+    def st_mtime_ns(self, val: int) -> None:
         """Set the modification time of the fake file in nanoseconds."""
         self._st_mtime_ns = val
 
     @property
-    def st_ctime_ns(self):
+    def st_ctime_ns(self) -> int:
         """Return the creation time in nanoseconds."""
         return self._st_ctime_ns
 
     @st_ctime_ns.setter
-    def st_ctime_ns(self, val):
+    def st_ctime_ns(self, val: int) -> None:
         """Set the creation time of the fake file in nanoseconds."""
         self._st_ctime_ns = val
 
@@ -296,7 +317,10 @@ class FakeStatResult:
 class BinaryBufferIO(io.BytesIO):
     """Stream class that handles byte contents for files."""
 
-    def putvalue(self, value):
+    def __init__(self, contents: Optional[bytes]):
+        super().__init__(contents or b'')
+
+    def putvalue(self, value: bytes) -> None:
         self.write(value)
 
 
@@ -304,13 +328,15 @@ class TextBufferIO(io.TextIOWrapper):
     """Stream class that handles Python string contents for files.
     """
 
-    def __init__(self, contents=None, newline=None, encoding=None,
-                 errors='strict'):
-        self._bytestream = io.BytesIO(contents)
+    def __init__(self, contents: Optional[bytes] = None,
+                 newline: Optional[str] = None,
+                 encoding: Optional[str] = None,
+                 errors: str = 'strict'):
+        self._bytestream = io.BytesIO(contents or b'')
         super().__init__(self._bytestream, encoding, errors, newline)
 
-    def getvalue(self):
+    def getvalue(self) -> bytes:
         return self._bytestream.getvalue()
 
-    def putvalue(self, value):
+    def putvalue(self, value: bytes) -> None:
         self._bytestream.write(value)
