@@ -81,17 +81,6 @@ class TestCase(unittest.TestCase):
             else:
                 self.assertEqual(subtype, exc.errno)
 
-    def assert_raises_os_error(self, subtype, expression, *args, **kwargs):
-        """Asserts that a specific subtype of OSError is raised."""
-        try:
-            expression(*args, **kwargs)
-            self.fail('No exception was raised, OSError expected')
-        except OSError as exc:
-            if isinstance(subtype, list):
-                self.assertIn(exc.errno, subtype)
-            else:
-                self.assertEqual(subtype, exc.errno)
-
 
 class RealFsTestMixin:
     """Test mixin to allow tests to run both in the fake filesystem and in the
@@ -132,9 +121,9 @@ class RealFsTestMixin:
 
     def set_windows_fs(self, value):
         if self.filesystem is not None:
-            self.filesystem.is_windows_fs = value
+            self.filesystem._is_windows_fs = value
             if value:
-                self.filesystem.is_macos = False
+                self.filesystem._is_macos = False
             self.create_basepath()
 
     @property
@@ -149,6 +138,9 @@ class RealFsTestMixin:
         """Return True if the real file system shall be tested."""
         return False
 
+    def setUpFileSystem(self):
+        pass
+
     def path_separator(self):
         """Can be overwritten to use a specific separator in the
         fake filesystem."""
@@ -177,7 +169,7 @@ class RealFsTestMixin:
                     'Testing Linux specific functionality')
         else:
             self.set_windows_fs(False)
-            self.filesystem.is_macos = False
+            self.filesystem._is_macos = False
 
     def check_macos_only(self):
         """If called at test start, the real FS test is executed only under
@@ -189,7 +181,7 @@ class RealFsTestMixin:
                     'Testing MacOS specific functionality')
         else:
             self.set_windows_fs(False)
-            self.filesystem.is_macos = True
+            self.filesystem._is_macos = True
 
     def check_linux_and_windows(self):
         """If called at test start, the real FS test is executed only under
@@ -201,7 +193,7 @@ class RealFsTestMixin:
                 raise unittest.SkipTest(
                     'Testing non-MacOs functionality')
         else:
-            self.filesystem.is_macos = False
+            self.filesystem._is_macos = False
 
     def check_case_insensitive_fs(self):
         """If called at test start, the real FS test is executed only in a
@@ -307,10 +299,12 @@ class RealFsTestMixin:
         args = [to_string(arg) for arg in args]
         return self.os.path.join(self.base_path, *args)
 
-    def create_dir(self, dir_path):
+    def create_dir(self, dir_path, perm=0o777):
         """Create the directory at `dir_path`, including subdirectories.
         `dir_path` shall be composed using `make_path()`.
         """
+        if not dir_path:
+            return
         existing_path = dir_path
         components = []
         while existing_path and not self.os.path.exists(existing_path):
@@ -325,6 +319,7 @@ class RealFsTestMixin:
             existing_path = self.os.path.join(existing_path, component)
             self.os.mkdir(existing_path)
             self.os.chmod(existing_path, 0o777)
+        self.os.chmod(dir_path, perm)
 
     def create_file(self, file_path, contents=None, encoding=None, perm=0o666):
         """Create the given file at `file_path` with optional contents,
@@ -362,7 +357,7 @@ class RealFsTestMixin:
         if self.filesystem is not None:
             old_base_path = self.base_path
             self.base_path = self.filesystem.path_separator + 'basepath'
-            if self.is_windows_fs:
+            if self.filesystem.is_windows_fs:
                 self.base_path = 'C:' + self.base_path
             if old_base_path != self.base_path:
                 if old_base_path is not None:
@@ -402,6 +397,17 @@ class RealFsTestMixin:
                               DummyTime(start, step))
         return DummyMock()
 
+    def assert_raises_os_error(self, subtype, expression, *args, **kwargs):
+        """Asserts that a specific subtype of OSError is raised."""
+        try:
+            expression(*args, **kwargs)
+            self.fail('No exception was raised, OSError expected')
+        except OSError as exc:
+            if isinstance(subtype, list):
+                self.assertIn(exc.errno, subtype)
+            else:
+                self.assertEqual(subtype, exc.errno)
+
 
 class RealFsTestCase(TestCase, RealFsTestMixin):
     """Can be used as base class for tests also running in the real
@@ -426,9 +432,6 @@ class RealFsTestCase(TestCase, RealFsTestMixin):
     def tearDown(self):
         RealFsTestMixin.tearDown(self)
 
-    def setUpFileSystem(self):
-        pass
-
     @property
     def is_windows_fs(self):
         if self.use_real_fs():
